/**********************************************************************
zyGrib: meteorological GRIB file viewer
Copyright (C) 2008-2010 - Jacques Zaninetti - http://www.zygrib.org

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
***********************************************************************/

#include "DataColors.h"

//--------------------------------------------------------------------
DataColors::DataColors ()
{
	initializeColors ();
	function_getColor = &DataColors::getWindColor;	// why not
}

//--------------------------------------------------------------------
void DataColors::initializeColors ()
{
	setCloudsColorMode ("cloudsColorMode");	
	mapColorTransp = 200;
//mapColorTransp = 255;
	
    // Color scale for wind in beaufort
    windColor[ 0].setRgba(qRgba(   0,  80, 255,  mapColorTransp));
    windColor[ 1].setRgba(qRgba(   0, 150, 255,  mapColorTransp));
    windColor[ 2].setRgba(qRgba(   0, 200, 255,  mapColorTransp));
    windColor[ 3].setRgba(qRgba(   0, 250, 180,  mapColorTransp));
    windColor[ 4].setRgba(qRgba(   0, 230, 150,  mapColorTransp));
    windColor[ 5].setRgba(qRgba( 255, 255,   0,  mapColorTransp));
    windColor[ 6].setRgba(qRgba( 255, 220,   0,  mapColorTransp));
    windColor[ 7].setRgba(qRgba( 255, 180,   0,  mapColorTransp));
    windColor[ 8].setRgba(qRgba( 255, 120,   0,  mapColorTransp));
    windColor[ 9].setRgba(qRgba( 230, 120,   0,  mapColorTransp));
    windColor[10].setRgba(qRgba( 220,  80,   0,  mapColorTransp));
    windColor[11].setRgba(qRgba( 200,  50,  30,  mapColorTransp));
    windColor[12].setRgba(qRgba( 170,   0,  50,  mapColorTransp));
    windColor[13].setRgba(qRgba( 150,   0,  30,  mapColorTransp));
    // Color scale for rain in mm/h
    rainColor[ 0].setRgba(qRgba(255,255,255,  mapColorTransp));
    rainColor[ 1].setRgba(qRgba(200,240,255,  mapColorTransp));
    rainColor[ 2].setRgba(qRgba(150,240,255,  mapColorTransp));
    rainColor[ 3].setRgba(qRgba(100,200,255,  mapColorTransp));
    rainColor[ 4].setRgba(qRgba( 50,200,255,  mapColorTransp));
    rainColor[ 5].setRgba(qRgba(  0,150,255,  mapColorTransp));
    rainColor[ 6].setRgba(qRgba(  0,100,255,  mapColorTransp));
    rainColor[ 7].setRgba(qRgba(  0, 50,255,  mapColorTransp));
    rainColor[ 8].setRgba(qRgba( 50,  0,255,  mapColorTransp));
    rainColor[ 9].setRgba(qRgba(100,  0,255,  mapColorTransp));
    rainColor[10].setRgba(qRgba(150,  0,255,  mapColorTransp));
    rainColor[11].setRgba(qRgba(200,  0,255,  mapColorTransp));
    rainColor[12].setRgba(qRgba(250,  0,255,  mapColorTransp));
    rainColor[13].setRgba(qRgba(200,  0,200,  mapColorTransp));
    rainColor[14].setRgba(qRgba(150,  0,150,  mapColorTransp));
    rainColor[15].setRgba(qRgba(100,  0,100,  mapColorTransp));
    rainColor[16].setRgba(qRgba( 50,  0,50,  mapColorTransp));
}
//--------------------------------------------------------------------------
QRgb  DataColors::getRainColor(double v, bool smooth)
{
    QRgb rgb = 0;
    int  ind;
    
    double indf = cbrt(67.5*v);        // TODO better color map!!!!
	if (v > 0)
		indf += 0.2;    
    
    ind = (int) floor(Util::inRange(indf, 0.0, 15.0));
    
    if (smooth && ind<16) {
        // Interpolation de couleur
        QColor c1 = rainColor[ind];
        QColor c2 = rainColor[ind+1];
        double dcol = indf-ind;
        rgb = qRgba(
            (int)( (double) c1.red()  *(1.0-dcol) + dcol*c2.red()   +0.5),
            (int)( (double) c1.green()*(1.0-dcol) + dcol*c2.green() +0.5),
            (int)( (double) c1.blue() *(1.0-dcol) + dcol*c2.blue()  +0.5),
            mapColorTransp
            );
    }
    else {
		ind = (int) (indf + 0.5);
		ind = Util::inRange(ind, 0, 15);
        rgb = rainColor[ind].rgba();
    }
    return rgb;
}
//--------------------------------------------------------------------------
QRgb  DataColors::getSnowDepthColor(double mm, bool smooth)
{
    QRgb rgb = 0;
    int  ind;
    double v = mm / 3.0;
    double indf = cbrt(900*v);        // TODO better color map!!!!
	if (v > 0)
		indf += 0.2;    
    ind = (int) floor(Util::inRange(indf, 0.0, 15.0));
    
    if (smooth && ind<16) {
        // Interpolation de couleur
        QColor c1 = rainColor[ind];
        QColor c2 = rainColor[ind+1];
        double dcol = indf-ind;
        rgb = qRgba(
            (int)( (double) c1.red()  *(1.0-dcol) + dcol*c2.red()   +0.5),
            (int)( (double) c1.green()*(1.0-dcol) + dcol*c2.green() +0.5),
            (int)( (double) c1.blue() *(1.0-dcol) + dcol*c2.blue()  +0.5),
            mapColorTransp
            );
    }
    else {
		ind = (int) (indf + 0.5);
		ind = Util::inRange(ind, 0, 15);
        rgb = rainColor[ind].rgba();
    }
    return rgb;
}
//--------------------------------------------------------------------------
QRgb  DataColors::getCloudColor(double v, bool smooth) {
    QRgb rgb = 0;
    if (!smooth) {
        v = 10.0*floor(v/10.0);
    }
	v = v*v*v/10000;
    int r,g,b;
    int tr;
    if (isCloudsColorModeWhite)
    {
		double k = 2.55;
		r = (int)(k*v);
		g = (int)(k*v);
		b = (int)(k*v);
		tr = (int)(2.5*v);
	}
	else {
		r = 255 - (int)(1.6*v);
		g = 255 - (int)(1.6*v);
		b = 255 - (int)(2.0*v);
		tr = mapColorTransp;
	}
    rgb = qRgba(r,g,b,  tr);
    return rgb;
}
//--------------------------------------------------------------------------
QRgb  DataColors::getDeltaTemperaturesColor(double v, bool smooth) {
	v = 100.0 - Util::inRange(20.0*v, 0.0, 100.0);
    QRgb rgb = 0;
    if (!smooth) {
        v = 10.0*floor(v/10.0);
    }
    int r,g,b;
    int tr;
		r = 255 - (int)(1.5*v);
		g = 255 - (int)(1.5*v);
		b = 255 - (int)(2.2*v);
		tr = mapColorTransp;
    rgb = qRgba(r,g,b,  tr);
    return rgb;
}
//--------------------------------------------------------------------------
QRgb  DataColors::getAltitudeColor(double v, const Altitude &alt, bool smooth) {
	double k=0.001, vmin=0, vmax=20000, vmoy=10000;
	if (alt == Altitude(LV_ISOTHERM0,0)) {
		vmin = 0;  vmax = 5000;
	}
	else if (alt.levelType == LV_ISOBARIC) {
		switch (alt.levelValue) {
			case 200 :
				vmoy = 11800; k=0.06; break;
			case 300 :
				vmoy = 9200; k=0.08; break;
			case 500 :
				vmoy = 5600; k=0.10;  break;
			case 700 :
				vmoy = 3000; k=0.15; break;
			case 850 :
				vmoy = 1460; k=0.25; break;
		}
		vmin = vmoy - k*vmoy;
		vmax = vmoy + k*vmoy;
	}
	return pasteToWindColorScale (v, vmin, vmax, smooth);
}
//--------------------------------------------------------------------------
QRgb  DataColors::getHumidColor(double v, bool smooth) {
    QRgb rgb = 0;
    if (!smooth) {
        v = 10.0*floor(v/10.0);
    }
    v = v*v/100;
    int r = 255 - (int)(2.4*v);
    int g = 255 - (int)(2.4*v);
    int b = 255 - (int)(1.2*v);
    rgb = qRgba(r,g,b,  mapColorTransp);
    return rgb;
}
//--------------------------------------------------------------------------
QRgb  DataColors::getCAPEColor(double v, bool smooth)
{
	double x = sqrt(v)*70.71;
	double t0 = 0;  // valeur mini de l'échelle
	double t1 = 4000;  // valeur maxi de l'échelle
	return pasteToWindColorScale (x, t0, t1, smooth);
}
//--------------------------------------------------------------------------
QRgb  DataColors::getTemperatureColor(double v, bool smooth)
{
	// Même échelle colorée que pour le vent
	double x = v-273.15;
	double t0 = -30;  // valeur mini de l'échelle
	double t1 =  40;  // valeur maxi de l'échelle
	return pasteToWindColorScale (x, t0, t1, smooth);
}
//--------------------------------------------------------------------------
QRgb  DataColors::getPressureColor(double v, bool smooth)
{
	// Même échelle colorée que pour le vent
	double x = v/100.0;	// Pa->hPa
	double t0 = 960;  // valeur mini de l'échelle
	double t1 = 1050;  // valeur maxi de l'échelle
	return pasteToWindColorScale (x, t0, t1, smooth);
}
//--------------------------------------------------------------------------
QRgb  DataColors::pasteToWindColorScale 
						(double v, double min, double max, bool smooth)
{
	double b0 = 0;    // min beauforts
	double b1 = 12;   // max beauforts
	double eqbeauf = b0 + (v-min)*(b1-b0)/(max-min);
	if (eqbeauf < 0)
		eqbeauf = 0;
	else if (eqbeauf > 12)
		eqbeauf = 12;
	return getWindColor (Util::BeaufortToKmh_F(eqbeauf), smooth);
}
//--------------------------------------------------------------------------
QRgb  DataColors::getWindColor(double v, bool smooth) 
{
    QRgb rgb = 0;
    int beauf;
    if (! smooth) {
        beauf = Util::kmhToBeaufort(v);
        rgb = windColor[beauf].rgba();
    }
    else {
        // Interpolation de couleur
        double fbeauf = Util::kmhToBeaufort_F(v);
        QColor c1 = windColor[ (int) fbeauf ];
        QColor c2 = windColor[ (int) fbeauf +1 ];
        double dcol = fbeauf-floor(fbeauf);
        rgb = qRgba(
				(int)( c1.red()  *(1.0-dcol) + dcol*c2.red()   +0.5),
				(int)( c1.green()*(1.0-dcol) + dcol*c2.green() +0.5),
				(int)( c1.blue() *(1.0-dcol) + dcol*c2.blue()  +0.5),
				mapColorTransp );
    }
    return rgb;
}
 
//--------------------------------------------------------------------------
void DataColors::setColorDataTypeFunction (const DataCode &dtc)
{
	switch (dtc.dataType) {
		case GRB_WIND_XY2D :
			function_getColor = &DataColors::getWindColor;
			break;
		case GRB_DIFF_TEMPDEW :
			function_getColor = &DataColors::getDeltaTemperaturesColor;
			break;
		case GRB_CLOUD_TOT : 
			function_getColor = &DataColors::getCloudColor;
			break;
		case GRB_PRECIP_TOT :
			function_getColor = &DataColors::getRainColor;
			break;
		case GRB_HUMID_REL :
			function_getColor = &DataColors::getHumidColor;
			break;
		case GRB_TEMP :
			function_getColor = &DataColors::getTemperatureColor;
			break;
		case GRB_TEMP_POT :
			function_getColor = &DataColors::getTemperatureColor;
			break;
		case GRB_DEWPOINT :
			function_getColor = &DataColors::getTemperatureColor;
			break;
		case GRB_SNOW_DEPTH :
			function_getColor = &DataColors::getSnowDepthColor;
			break;
		case GRB_SNOW_CATEG :
			function_getColor = &DataColors::getSnowDepthColor;
			break;
		case GRB_FRZRAIN_CATEG :
			function_getColor = &DataColors::getSnowDepthColor;
			break;
		case GRB_CAPE :
			function_getColor = &DataColors::getCAPEColor;
			break;
		default :
			function_getColor = &DataColors::getWindColor;	// why not
			break;
	}
}
 
//--------------------------------------------------------------------------
QRgb DataColors::getDataCodeColor (const DataCode &dtc, double v, bool smooth)
{
	switch (dtc.dataType) {
		case GRB_WIND_XY2D :
			return DataColors::getWindColor (v, smooth);
			break;
		case GRB_DIFF_TEMPDEW :
			return DataColors::getDeltaTemperaturesColor (v, smooth);
			break;
		case GRB_CLOUD_TOT : 
			return DataColors::getCloudColor (v, smooth);
			break;
		case GRB_PRECIP_TOT :
			return DataColors::getRainColor (v, smooth);
			break;
		case GRB_HUMID_REL :
			return DataColors::getHumidColor (v, smooth);
			break;
		case GRB_TEMP :
			return DataColors::getTemperatureColor (v, smooth);
			break;
		case GRB_TEMP_POT :
			return DataColors::getTemperatureColor (v, smooth);
			break;
		case GRB_DEWPOINT :
			return DataColors::getTemperatureColor (v, smooth);
			break;
		case GRB_SNOW_DEPTH :
			return DataColors::getSnowDepthColor (v, smooth);
			break;
		case GRB_SNOW_CATEG :
			return DataColors::getSnowDepthColor (v, smooth);
			break;
		case GRB_FRZRAIN_CATEG :
			return DataColors::getSnowDepthColor (v, smooth);
			break;
		case GRB_CAPE :
			return DataColors::getCAPEColor (v, smooth);
			break;
		default :
			return DataColors::getWindColor (v, smooth);	// why not
			break;
	}
}

//--------------------------------------------------------------------------
bool DataColors::getMapColorMinMaxStep (
			const DataCode &dtc, 
			double *min, double *max, double *step)
{
// DBG("type=%d", dtc.dataType);

	switch (dtc.dataType) {
		case GRB_WIND_XY2D :
			*min = 0;
			*max = 12;
			*step = 1;
			return true;

		case GRB_DIFF_TEMPDEW :
		case GRB_CLOUD_TOT : 
		case GRB_PRECIP_TOT :
		case GRB_HUMID_REL :
		case GRB_TEMP :
		case GRB_TEMP_POT :
		case GRB_DEWPOINT :
		case GRB_SNOW_DEPTH :
		case GRB_SNOW_CATEG :
		case GRB_FRZRAIN_CATEG :
		case GRB_CAPE :
		default :
			return false;
	}
}

					







