/*
 *  CurveDrawer.h
 *  zyGrib
 *
 *  Created by Tim Holtschneider on 29.05.10.
 *  Copyright 2010. All rights reserved.
 *
 */
#ifndef DATAPLOT_H
#define DATAPLOT_H
#include <cmath>
#include <QObject>

#include "GriddedPlotter.h"
#include "POI.h"
#include "DataPointInfo.h"
#include "ui_CurveDrawerDialog.h"
#include "CustomQwtClasses.h"

#include <qwt_plot_curve.h>
#include <qwt_scale_engine.h>
#include <qwt_plot_picker.h>
#include <qwt_legend_item.h>
#include <qwt_plot_marker.h>

//===================================================================
// DataPlotter : dialog
//===================================================================
class CurveDrawer: 
	public	QDialog,
	private	Ui_CurveDrawerDialog
{ Q_OBJECT
    public:
        CurveDrawer( GriddedPlotter *, 
					 double, double, QWidget * );
		CurveDrawer( GriddedPlotter *, 
					 double, double, double, double, QWidget *);

        ~CurveDrawer();
	
    private:
    	GriddedPlotter	*plotter;
		GriddedReader	*reader;
		double			lonFirst, latStart, lonStart, lonEnd, latEnd;
		time_t			tRefDate, tCurDate;

		// Lists for grib data
		QVector<QString> qvDates;
		QVector<double> qvHoursFromNow;
		QVector<double> qvDist;
		QVector<double> *qvXAxis;
		QVector<double> *qvY2Axis;
		QVector<double>	qvY1Axis;
		QVector<double> qvWindDir;
		QVector<double> qvWindDev;
		QVector<double> qvTempMin;
		QVector<double> qvTempMax;
		int iDataCnt;
	
		// Qwt objects
		QwtScaleEngine  *qwtYScale[2];
		QwtPlotCurve	*qwtCurve[4];
//		QwtPlotPicker	*qwtPicker;
		CustomQwtPicker *qwtPicker;
		QwtLegend		*qwtLegend;
		QwtPlotMarker	*qwtMarker[2];
	
		// methods to create and update plots
		bool	createTable( const int & );
		bool	createTablePOI( const int & );
		void	initDataPlot( const int & );
		void	createDataPlot( const int &, const QString & );
		void	activateLegend( QwtPlotItem *, bool );
		void	initInterface();
		bool	privateConstructor();
	
		void	closeEvent(QCloseEvent *) {delete this;};
		QString	getPlotUnit( const int & );
	
		double getXAxisStep( const double&, const double& );

	signals:
		void legendClicked( QwtPlotItem * );
	
	private slots:
		void slotExitClicked();
		void slotCurrentClicked(bool);
		void slotDataChosen(int);
		void slotLegendChecked( QwtPlotItem *, bool );
	
};

#endif
