/*
 *  CustomQwtClasses.cpp
 *  zyGrib
 *
 *  Created by Tim Holtschneider on 25.11.10.
 *
 */

#include "CustomQwtClasses.h"
//#include <QVector>
#include "Util.h"

/*
 *  Custom Scale Draw Class
 */
CustomQwtScale::CustomQwtScale( const QVector<double> &qvOrig, const QVector<double> &qvNew ){
	myOrig = qvOrig;
	myRepl = qvNew;
}

CustomQwtScale::CustomQwtScale( const QVector<double> &qvOrig, const QVector<QString> &qvNew ){
	myOrig    = qvOrig;
	myReplStr = qvNew;
}


QwtText CustomQwtScale::label( double val ) const {
	if (myOrig.contains( val ) ) {
		int iElement = myOrig.indexOf( val );
		
		if( myRepl.size() > 0)
		{
			return QString( "%1" ).arg( myRepl.at(iElement), 4, 's', 1 );
		} else if( myReplStr.size() > 0) {
			return myReplStr.at( iElement );
		}
	}
	return QString( "%1" ).arg( val, 4, 's', 1 );
}

/*
 *  Custom Plot Picker Class
 */
CustomQwtPicker::CustomQwtPicker( int xAxis, int yAxis, int selectionFlags, RubberBand rubberBand, 
								 DisplayMode trackerMode, QwtPlotCanvas *canvas, const QDateTime &dtCurrent ) :
	QwtPlotPicker( xAxis, yAxis, selectionFlags, rubberBand, trackerMode, canvas )
{
	dt = dtCurrent;
}

QwtText CustomQwtPicker::trackerText( const QwtDoublePoint &pos ) const {
	
	double dXPos = pos.x();
	QString sText;
	QDateTime myQTime = dt;
	
	// calulate hours to add to current time position from plot position
	uint iDays = (uint)dXPos % 24;
	dXPos -= iDays * 24;
	uint iSecs = (uint)(dXPos * 3600.);
	
	myQTime = myQTime.addSecs(iSecs);
	myQTime = myQTime.addDays(iDays);
	
	sText.sprintf("\n%.3f", pos.y() );
	sText.prepend( myQTime.toString("dd.MM. hh:mm") );
	return sText;
}
